# cody.py (Explorer Workshop)
# Copyright: 2024 Bradan Lane STUDIO
#
# Open Source License: MIT

'''
This CircuitPython program is a sample from the 'Hack the Badge' workshop

Implementation Notes
--------------------
**Hardware:**
    * one of the Explorer Badge designs

**Software and Dependencies:**
    * CircuitPython firmware (UF2): https://circuitpython.org
    * Adafruit CircuitPython Libraries: https://circuitpython.org/libraries

**Open Source Repository:**
    * GitLab: https://gitlab.com/bradanlane/exp_tutorial
    
'''

import board
import time
import digitalio
import audiopwmio
import audiobusio
import audiocore
import audiomixer

import neopixel
import touchio

USE_I2S_AUDIO = False


''' setup the Neopixel LEDs '''
NEOPIXEL_COUNT: int = 9
leds = neopixel.NeoPixel(board.NEOPIXEL, NEOPIXEL_COUNT, brightness=0.15, auto_write=True)

# lets quickly cycle all the neopixels through a range of colors
colors: list = [0x0000FF, 0x00FF00, 0x00FFFF, 0xFF0000, 0xFF00FF, 0xFFFF00, 0xFFFFFF, 0x000000]
COLOR_COUNT: int = 8
color_index: int = 0

''' setup the capacitive touch sensors '''
TOUCH_PINS: list = [board.TOUCH1, board.TOUCH2, board.TOUCH3, board.TOUCH4, board.TOUCH5, board.TOUCH6, board.TOUCH7, board.TOUCH8, board.TOUCH9]
TOUCH_COUNT:int = 9
touch_sensors = [None for i in range(TOUCH_COUNT)]

for i in range(TOUCH_COUNT):
    touch_sensors[i] = touchio.TouchIn(TOUCH_PINS[i])
    # the minimum sensitivity (aka threshold) is set when the touch sensor is created
    print(f"initialized touch {i:d} with threshold {touch_sensors[i].threshold:d}")


''' seetup and load sound WAV files '''
PAD_NAMES = ['kick', 'snare', 'hatclosed', 'hatopen', 'clap', 'tom']
sounds = [None] * TOUCH_COUNT # create a TOUCH_COUNT sized list of None values
active = [False] * TOUCH_COUNT # keeps track of which pad notes are active

mixer = audiomixer.Mixer(voice_count=TOUCH_COUNT, 
                         sample_rate=22050, 
                         channel_count=1, 
                         bits_per_sample=16, 
                         samples_signed=True, 
                         buffer_size=4096)

if USE_I2S_AUDIO:
    audio = audiobusio.I2SOut(board.I2S_BCK, board.I2S_LRCK, board.I2S_DATA)
    sound_enable = None
    for i in range(TOUCH_COUNT):
        mixer.voice[i].level = 0.15
else:
    audio = audiopwmio.PWMAudioOut(board.SPEAKER)
    sound_enable = digitalio.DigitalInOut(board.SPEAKER_EN)
    sound_enable.direction = digitalio.Direction.OUTPUT
    sound_enable.value = True # make sure the amplifier is ON

audio.play(mixer)


def load_sounds(folder: str):
    '''
        folder is a string and represents the full path to a set of files
        within the folder should be a series of numbered '.wav' files
        00kick, 01snare, 02hatclosed, 03hatopen, 04clap, 05tom
    '''
    for i in range(TOUCH_COUNT):
        try:
            fname = f"{folder}/{i:02d}{PAD_NAMES[i]}.wav"
            sounds[i] = audiocore.WaveFile(open(fname,"rb"))
        except Exception as e:
            print(f"Error while assigning wave audio #{i:d}:", e)
            sounds[i] = None

def play_sound(num: int, pressed: bool):
    num = num % TOUCH_COUNT # safety to prevent values out of range
    voice = mixer.voice[num]   # get mixer voice
    if pressed:
        voice.play(sounds[num],loop=False)
    else: # released
        pass   # not doing this for samples

# load one of the sets of sound files
load_sounds("/sounds/studio")	# studio quilcom tr919


try:
    while True:
        for i in range(TOUCH_COUNT):
            if touch_sensors[i].value:
                if not active[i]:
                    active[i] = True
                    leds[i] = 0x00FF00
                    if sounds[i]:
                        print(f"touch #{i:d} has a value of {touch_sensors[i].raw_value:d} and plays {PAD_NAMES[i]}")
                        play_sound(i, True)
            else:
                if active[i]:
                    active[i] = False
                    leds[i] = 0x000000
                    if sounds[i]:
                        play_sound(i, False)
        time.sleep(0.01)

finally:
    if sound_enable:
        sound_enable.value = False
    audio.deinit()
    leds.fill(0x000000)
